@@ -1031,7 +1031,7 @@ list_keyblock_print (KBNODE keyblock, int secret, int fpr,
 
 	      validity = uid_trust_string_fixed (pk, uid);
 	      indent =
-		(keystrlen () + 9) -
+		(keystrlen () + (opt.legacy_list_mode? 9:11)) -
 		atoi (uid_trust_string_fixed (NULL, NULL));
 
 	      if (indent < 0 || indent > 40)
@@ -1040,7 +1040,8 @@ list_keyblock_print (KBNODE keyblock, int secret, int fpr,
 	      es_fprintf (es_stdout, "uid%*s%s ", indent, "", validity);
 	    }
 	  else
-	    es_fprintf (es_stdout, "uid%*s", (int) keystrlen () + 10, "");
+	    es_fprintf (es_stdout, "uid%*s",
+                        (int) keystrlen () + (opt.legacy_list_mode? 10:12), "");
 
 	  print_utf8_buffer (es_stdout, uid->name, uid->len);
 	  es_putc ('\n', es_stdout);
@@ -1086,18 +1087,6 @@ list_keyblock_print (KBNODE keyblock, int secret, int fpr,
                   pubkey_string (pk2, pkstrbuf, sizeof pkstrbuf),
 		  keystr_from_pk (pk2), datestr_from_pk (pk2));
 
-          if (pk2->pubkey_algo == PUBKEY_ALGO_ECDSA
-              || pk2->pubkey_algo == PUBKEY_ALGO_EDDSA
-              || pk2->pubkey_algo == PUBKEY_ALGO_ECDH)
-            {
-              char *curve = openpgp_oid_to_str (pk2->pkey[0]);
-              const char *name = openpgp_oid_to_curve (curve);
-              if (!*name || *name == '?')
-                name = curve;
-              es_fprintf (es_stdout, " %s", name);
-              xfree (curve);
-            }
-
           if ((opt.list_options & LIST_SHOW_USAGE))
             {
               es_fprintf (es_stdout, " [%s]", usagestr_from_pk (pk2, 0));
@@ -1358,8 +1347,8 @@ list_keyblock_colon (KBNODE keyblock, int secret, int has_secret, int fpr)
       || pk->pubkey_algo == PUBKEY_ALGO_ECDH)
     {
       char *curve = openpgp_oid_to_str (pk->pkey[0]);
-      const char *name = openpgp_oid_to_curve (curve);
-      if (!*name || *name == '?')
+      const char *name = openpgp_oid_to_curve (curve, 0);
+      if (!name)
         name = curve;
       es_fputs (name, es_stdout);
       xfree (curve);
@@ -1487,8 +1476,8 @@ list_keyblock_colon (KBNODE keyblock, int secret, int has_secret, int fpr)
               || pk->pubkey_algo == PUBKEY_ALGO_ECDH)
             {
               char *curve = openpgp_oid_to_str (pk->pkey[0]);
-              const char *name = openpgp_oid_to_curve (curve);
-              if (!*name || *name == '?')
+              const char *name = openpgp_oid_to_curve (curve, 0);
+              if (!name)
                 name = curve;
               es_fputs (name, es_stdout);
               xfree (curve);
@@ -1725,6 +1714,7 @@ print_icao_hexdigit (estream_t fp, int c)
  *      1: print using log_info ()
  *      2: direct use of tty
  *      3: direct use of tty but only primary key.
+ *      4: direct use of tty but only subkey.
  *     10: Same as 0 but with_colons etc is ignored.
  *
  * Modes 1 and 2 will try and print both subkey and primary key
@@ -1783,7 +1773,7 @@ print_fingerprint (estream_t override_fp, PKT_public_key *pk, int mode)
     {
       fp = override_fp; /* Use tty or given stream.  */
       if (primary)
-	/* TRANSLATORS: this should fit into 24 bytes to that the
+	/* TRANSLATORS: this should fit into 24 bytes so that the
 	 * fingerprint data is properly aligned with the user ID */
 	text = _(" Primary key fingerprint:");
       else
@@ -1794,6 +1784,11 @@ print_fingerprint (estream_t override_fp, PKT_public_key *pk, int mode)
       fp = override_fp; /* Use tty or given stream.  */
       text = _("      Key fingerprint =");
     }
+  else if (mode == 4)
+    {
+      fp = override_fp; /* Use tty or given stream.  */
+      text = _("      Subkey fingerprint:");
+    }
   else
     {
       fp = override_fp? override_fp : es_stdout;
